import os
import platform
import socket
import sys
import typing

import click

from ytservice import daemon

# Constants for response codes
RESULT_CODE_ERROR = 0
RESULT_CODE_SUCCESS = 1


def get_local_ip():
    try:
        sock = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        sock.connect(("8.8.8.8", 80))
        local_ip = sock.getsockname()[0]
        return local_ip
    except socket.error:
        return "127.0.0.1"


def is_mac() -> bool:
    """
    is Mac
    :return:
    """
    return platform.system() == "Darwin"


# Helper function to build a standard response dictionary
def build_response(code: int = RESULT_CODE_SUCCESS,
                   data: typing.Any = None,
                   message: str = None) -> dict:
    response = {"code": code}
    if data is not None:
        response["data"] = data
    if message:
        response["message"] = message
    return response


def run_service(app_name: str, signal: str, out_path: str = "/dev/null",
                error_path: str = "/dev/null"):
    """
    run service
    """
    if not is_mac():
        click.echo(message="Not Support")
        return
    try:
        cmd_path = os.path.abspath(sys.argv[0])
        if cmd_path.endswith(".py"):
            click.echo(message="Not Support", err=True)
            return
        if signal == "install":
            cmd = [cmd_path]
            environment = {"PATH": "/opt/homebrew/bin:/usr/local/bin"}
            daemon.mac_install(name=app_name, program_arguments=cmd, out_path=out_path, error_path=error_path,
                               environment_variables=environment)
            click.secho("Done", fg="green")
        elif signal == "start":
            daemon.mac_start(name=app_name)
            click.secho("Done", fg="green")
        elif signal == "stop":
            daemon.mac_stop(name=app_name)
            click.secho("Done", fg="green")
        elif signal == "status":
            is_install = daemon.mac_is_install(name=app_name)
            if is_install:
                is_running = daemon.mac_is_running(name=app_name)
                if is_running:
                    click.secho("Running", fg="green")
                else:
                    click.secho("Stopped", fg="red")
            else:
                click.secho("Not Install", fg="red")
        elif signal == "rm":
            daemon.mac_rm(name=app_name)
            click.secho("Done", fg="green")
        else:
            pass
    except RuntimeError as e:
        click.secho(e, fg="red")



# -*- coding: utf-8 -*-

import os
import plistlib

PREFIX_LABEL = "org.seven"
CMD_LOAD = "launchctl load -w {}"
CMD_UNLOAD = "launchctl unload -w {}"
CMD_LIST = "launchctl list"


def launch_agents() -> str:
    """
    launch agents
    """
    agents = os.path.join(os.path.expanduser("~"), "Library", "LaunchAgents")
    if not os.path.exists(agents):
        os.mkdir(agents)
    return agents


def _plist_file(name: str) -> (str, str):
    """
    plist file
    """
    label = "{}.{}".format(PREFIX_LABEL, name)
    plist = os.path.join(launch_agents(), label + ".plist")
    return label, plist


def mac_start(name: str) -> None:
    if not mac_is_install(name=name):
        raise RuntimeError("Not Install")
    running = mac_is_running(name=name)
    if not running:
        _, plist = _plist_file(name=name)
        os.system(CMD_LOAD.format(plist))


def mac_stop(name: str) -> None:
    if not mac_is_install(name=name):
        raise RuntimeError("Not Install")
    running = mac_is_running(name=name)
    if running:
        _, plist = _plist_file(name=name)
        os.system(CMD_UNLOAD.format(plist))


def mac_rm(name: str) -> None:
    """
    mac rm
    """
    if mac_is_install(name=name):
        if mac_is_running(name=name):
            mac_stop(name=name)
        _, plist = _plist_file(name=name)
        os.remove(plist)


def mac_is_install(name: str) -> bool:
    """
    is install
    """
    _, plist = _plist_file(name=name)
    return os.path.exists(plist)


def mac_is_running(name: str) -> bool:
    """
    is running
    """
    l, _ = _plist_file(name=name)
    result = False
    cmd_result = os.popen(cmd=CMD_LIST).readlines()
    if cmd_result and len(cmd_result) > 0:
        for line in cmd_result:
            item = line.strip().split("\t")
            if len(item) == 3:
                pid = item[0].strip()
                label = item[2].strip()
                if l == label and pid != "-":
                    result = int(pid) > 0
    return result


def mac_install(name: str,
                out_path: str,
                error_path: str,
                program_arguments: list = None,
                keep_alive: bool = True,
                run_at_load: bool = True,
                working_dir: str = None,
                environment_variables: dict = None) -> None:
    """
    plist install
    :param name:
    :param program_arguments:
    :param out_path:
    :param error_path:
    :param keep_alive:
    :param run_at_load:
    :param working_dir:
    :return:
    """
    if not name:
        raise ValueError("label empty")
    if not program_arguments:
        raise ValueError("program arguments empty")
    label, plist = _plist_file(name=name)
    if os.path.exists(plist):
        raise RuntimeError("{0} - exists".format(label))

    pl = dict(
        Label=label,
        ProgramArguments=program_arguments,
        KeepAlive=keep_alive,
        RunAtLoad=run_at_load,
        StandardOutPath=out_path,
        StandardErrorPath=error_path
    )
    if working_dir:
        pl["WorkingDirectory"] = working_dir
    if environment_variables:
        pl["EnvironmentVariables"] = environment_variables

    with open(plist, "wb") as f:
        plistlib.dump(pl, f)



from datetime import datetime

import yt_dlp
from fastapi import HTTPException

from ytservice.model import Settings


# Generate yt-dlp options for video downloading
def generate_video_options(settings: Settings) -> dict:
    ydl_opts = {
        'quiet': True,
        'ignoreerrors': True,
        'format': 'bestvideo[ext=mp4]+bestaudio[ext=m4a]/best[ext=mp4]',
        'noplaylist': True,
        'merge_output_format': 'mp4',
        'postprocessors': [{'key': 'FFmpegCopyStream'}],
        'postprocessor_args': {'copystream': ['-c:v', 'libx264', '-c:a', 'aac']},
        'buffer_size': '128M',
        'n_threads': 16,
    }

    # ydl_opts = {
    #     'quiet': True,
    #     'ignoreerrors': True,
    #     'format': 'bestvideo[ext=mp4][protocol!*=m3u8]+bestaudio[ext=m4a]/best[ext=mp4]',
    #     'noplaylist': True,
    #     'merge_output_format': 'mp4',
    #     'buffer_size': '128M',
    #     'n_threads': 16,
    # }

    if settings.proxy:
        ydl_opts['proxy'] = settings.proxy

    settings.download_dir.mkdir(parents=True, exist_ok=True)
    ydl_opts['outtmpl'] = str(settings.download_dir / datetime.now().strftime('%Y%m%d%H%M%S') / '%(title)s.%(ext)s')

    return ydl_opts


# Generate yt-dlp options for audio downloading
def generate_audio_options(settings: Settings) -> dict:
    ydl_opts = {
        'quiet': True,
        'ignoreerrors': True,
        'format': 'bestaudio[ext=m4a]',
        'noplaylist': True
    }

    if settings.proxy:
        ydl_opts['proxy'] = settings.proxy

    settings.download_dir.mkdir(parents=True, exist_ok=True)
    ydl_opts['outtmpl'] = str(settings.download_dir / datetime.now().strftime('%Y%m%d%H%M%S') / '%(title)s.%(ext)s')

    return ydl_opts


def optimize_audio_info(url: str, settings: Settings):
    info = extract_audio_info(url=url, settings=settings)
    best_info = dict(
        title=info.get("title", ""),
        extractor=info.get("extractor", ""),
        url=info.get("url", ""),
        ext=info.get("ext", ""),
        format=info.get("format", ""),
        resolution=info.get("resolution", ""))
    return best_info


def optimize_video_info(url: str, settings: Settings):
    info = extract_video_info(url=url, settings=settings)
    requested_formats = info.get("requested_formats", [])
    if not requested_formats:
        raise HTTPException(status_code=500, detail="No requested formats found")

    best_info = dict(
        title=info.get("title", ""),
        extractor=info.get("extractor", ""),
        requested_formats=[])

    for requested in requested_formats:
        mats = dict(
            url=requested.get("url", ""),
            ext=requested.get("ext", ""),
            format=info.get("format", ""),
            resolution=requested.get("resolution", ""))
        best_info["requested_formats"].append(mats)

    return best_info


def extract_audio_info(url: str, settings: Settings) -> dict:
    ydl_opts = generate_audio_options(settings=settings)
    return _extract_info(url=url, ydl_opts=ydl_opts)


def extract_video_info(url: str, settings: Settings) -> dict:
    ydl_opts = generate_video_options(settings=settings)
    return _extract_info(url=url, ydl_opts=ydl_opts)


# Extract video information without downloading
def _extract_info(url: str, ydl_opts: dict) -> dict:
    with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        return ydl.extract_info(url=url, download=False)


# Download video and return the file path
def download_video(url: str, settings: Settings) -> dict:
    ydl_opts = generate_video_options(settings=settings)
    download_info = {}

    # Progress hook to capture the file path after download
    def progress_hook(resp):
        if resp['status'] == 'finished':
            download_info["url"] = (resp['info_dict']['filename'])
            download_info["title"] = (resp['info_dict']['title'])
            download_info["description"] = (resp['info_dict']['description'])
            download_info["extractor"] = (resp['info_dict']['extractor'])
            download_info["format"] = (resp['info_dict']['format'])
            download_info["ext"] = (resp['info_dict']['ext'])
            download_info["resolution"] = (resp['info_dict']['resolution'])

    ydl_opts['progress_hooks'] = [progress_hook]

    with yt_dlp.YoutubeDL(ydl_opts) as ydl:
        ydl.download([url])

    return download_info


if __name__ == '__main__':
    print("test")
